#include "bocon.h"
#include "globprep.h"
#include "globalt.h"
#include "intools.h"
#include "mathem.h"
#include <string.h>

bocon::bocon()
{
   ndir =  0L;
   dir  = NULL;
   tf   = NULL;
   neq  = NULL;
   f    = NULL;
   eq   = NULL;
   ft   = NULL;
   tabf = NULL;
   lv   = NULL;
   iv   = NULL;
}

bocon::~bocon()
{
  for (long i = 0; i < ndir; i++)
  {
    delete [] lv[i];
    delete tabf[i];
    for (long j = 0; j < neq[i]; j++)
    {
      delete eq[i][j];
      delete [] ft[i][j];
    }
  }
  delete [] dir;
  delete [] tf;
  delete [] neq;
  delete [] f;

  delete [] eq;
  delete [] ft;
  delete [] lv;
  delete [] tabf;
  delete [] iv;
}

long bocon::read(FILE *in, long ndofn)
{
  long i, j, tnd, td;
  Parser par;

  getlong(in, tnd);
  if ((tnd < 1) && (tnd > ndofn))
    return 1;
  
  ndir = ndofn;
  dir  = new long [ndofn];
  for (i = 0; i < ndofn; i++)
    dir[i] = -1;
  tf   = new generalfunct [ndofn];
  for (i = 0; i < ndofn; i++)
    tf[i] = generalfunct(-1);
  neq  = new long [ndofn];
  memset(neq, 0, sizeof(*neq)*ndofn);
  f    = new double [ndofn];
  memset(f, 0, sizeof(*f)*ndofn);
  eq   = new Equation **[ndofn];
  memset(eq, 0, sizeof(*eq)*ndofn);
  ft   = new char **[ndofn];
  memset(ft, 0, sizeof(*ft)*ndofn);
  tabf  = new tablefunct *[ndofn];
  memset(tabf, 0, sizeof(*tabf)*ndofn);
  lv   = new double *[ndofn];
  memset(lv, 0, sizeof(*lv)*ndofn);
  switch (Tp->tprob)
  {
    case stationary_problem :
      break;
    case nonstationary_problem :
    case nonlinear_nonstationary_problem :
      iv = new double [ndofn];
      break;
    default :
      fprintf(stderr, "\n\n Unknown type of problem required in function bocon::read\n");
      fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
  }

  for (i = 0; i < tnd; i++)
  {
    getlong(in, td);
    if ((td < 1) && (td > ndofn))
      return 1;
    td--;
    dir[td] = td;
    switch (Tp->tprob)
    {
      case stationary_problem :
        break;
      case nonstationary_problem :
      case nonlinear_nonstationary_problem :
        getdouble(in, iv[td]);
        break;
      default :
        fprintf(stderr, "\n\n Unknown type of problem required in function bocon::read\n");
        fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
    }
    getint(in, (int&)tf[td]);
    switch (tf[td])
    {
      case stat :
        break;
      case pars :
        neq[td] = 1;
        eq[td] = new Equation *[neq[td]];
        ft[td] = new char *[neq[td]];
        ft[td][0] = new char[256];
        break;
      case tab  :
        tabf[td] = new tablefunct;
        getlong(in, j);
        tabf[td]->asize = j;
        tabf[td]->x = new double[j];
        tabf[td]->y = new double[j];
        getlong(in, j);
        tabf[td]->itype = interpoltype(j);
        break;
      case pars_set :
        getlong(in, neq[td]);
        eq[td] = new Equation *[neq[td]];
        ft[td] = new char *[neq[td]];
        lv[td] = new double[neq[td]];
        for (j = 0; j < neq[td]; j++)
          ft[td][j] = new char[256];
        break;
      default :
        fprintf(stderr, "\n\n Unknown type of time function is required in function bocon::read().\n");
        fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
        return(2);
    }
    switch (tf[td])
    {
      case stat :
        getdouble(in, f[td]);
        break;
      case pars :
        getstring(in, ft[td][0], 255);
        eq[td][0] = par.TextToTree(ft[td][0]);
        if (eq[td][0] == NULL)
        {
          fprintf(stderr, "\n\n Unable to parse function bocon::read().\n");
          fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
          return (3);
        }
        break;
      case tab  :
        for (j = 0; j < tabf[td]->asize; j++)
        {
          getdouble(in, tabf[td]->x[i]);
          getdouble(in, tabf[td]->y[i]);
        }
        break;
      case pars_set :
        for (j = 0; j < neq[td]; j++)
        {
          getdouble(in, lv[td][j]);
          getstring(in, ft[td][j], 255);
          eq[td][j] = par.TextToTree(ft[td][j]);
          if (eq[td][j] == NULL)
          {
            fprintf(stderr, "\n\n Unable to parse function bocon::read().\n");
            fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
            return (3);
          }
        }
        break;
      default :
        fprintf(stderr, "\n\n Unknown type of time function is required in function bocon::read().\n");
        fprintf(stderr, "(file %s, line %d)\n", __FILE__, __LINE__);
        return (2);
    }
  }
  return(0);
}
