#include "dbmat.h"
#include "../../GEFEL/intools.h"
#include <string.h>

matr :: matr(void)
{
  type = mattypet(-1);
  nprop = 0;
  prop = NULL;
  propu = NULL;
  ridx = NULL;
  npropu = 0;
}

matr :: ~matr(void)
{
  for (long i=0; i < nprop; i++)
    delete [] prop[i];
  delete [] prop;
  delete [] propu;
  delete [] ridx;
}

long matr :: read(FILE *in)
{

  if (getint(in, (int &)type) || getlong(in, nprop))
  // reading material type number and number of indeces
  {
    fprintf(stderr, "\nError reading material parameters (type, num)\n");
    return(1);
  }
  // allocating apropriate ammount of memory
  prop = new char*[nprop];
  memset(prop, 0, sizeof(*prop)*nprop);
  propu = new long[nprop];
  memset(propu, 0, sizeof(*propu)*nprop);
  ridx = new long[nprop];
  memset(ridx, 0, sizeof(*ridx)*nprop);
  for (long i=0; i < nprop; i++)

  // Reading strings with material properties for each material index
  //  each line in the input file corresponds to one string with material
  //  properties of one index
  {
    prop[i] = new char[1025];
    if (getstring(in, prop[i], 1024))
    {
      fprintf(stderr, "\nError reading material parameters (properties string)\n");
      return(2);
    }
  }
  for (long i = 0; i < nprop; i++)
  {
    if (prop[i][strlen(prop[i])-1] == '\n')
    // replace last character with 0 if equal newline
      prop[i][strlen(prop[i])-1] = '\0';
  }
  return(0);
}



dbmat :: dbmat(void)
{
  numt = 0;
  mat = NULL;
}

dbmat :: ~dbmat(void)
{
  delete [] mat;
}

long dbmat :: search_mat(mattypet ce, long ci)
// Function searches database whether contains material with
//  type number ce and type index number ci.
// Parameters :
//  ce - material type number
//  ci - material type index
// Returns :
//  If the material with given parameters exists in the database function returns
//  index of given material in the array mat else returns -1.
{
  long ret = -1;
  for (long i = 0; i < numt; i++)
  {
    if ((mat[i].type == ce) && ((ci >= 0) && (ci < mat[i].nprop)))
    {
      ret = i;
      break;
    }
  }
  return(ret);
}
