#define EXTERN
#include "globalt.h"
#include "globmatt.h"
#include "genfile.h"
#include "input.h"
#include "output.h"
#include "globprep.h"

#include <stdio.h>

int main (int argc,char *argv[])
{
  XFILE *in;
  long err;
  descrip *d;

  fprintf (stderr,"\n\n *** TRFEL PREPROCESSOR ***\n");
  fprintf (stderr," --------------------------\n");

  Tp = new probdesct;
  Gtt = new gtopology;
  Tt = new transtop;
  Tm = new transmat;
  Tc = new transcrsec;
  Tb = new transbclc;
  Outdt = new outdrivert;


  if (argc < 3){
    fprintf (stderr,"\n Wrong number of command line parameters.");
    fprintf (stderr,"\n Use : transprep input_file_name output_file_name\n\n");
    delete Tp;  delete Tt;  delete Tm;  delete Tc;  delete Tb;
    return(1);
  }
  in = xfopen(argv[1],"r");
  if (in==NULL){
    fprintf (stderr,"\n Input file has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    delete Tp;  delete Tt;  delete Tm;  delete Tc;  delete Tb;
    return(2);
  }
  in->maxlnsize = 1024;
  in->warrning = 1;
  in->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  in->ignorecase = 1;


  d = new descrip;
  // reading line with topology file name
  xfscanf(in, "%a", d->t3df);
  // reading line with material database file name
  xfscanf(in, "%a", d->matf);
  // reading line with cross-section database file name
  xfscanf(in, "%a", d->crf);
  // reading line with topology file format indicator,
  xfscanf(in, "%ld", &d->t3d);
  d->seq = 1;
  // reading line with edge number indicator
  xfscanf(in, "%ld", &d->redgn);
  // reading problem description
  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "begin");
  in->kwdmode = ignore;
  Tp->read(in);

  Lbt   = new linbart;
  Qbt   = new quadbart;
  Ltt   = new trlineart;
  Ltat  = new trlinaxisym;
  Lqt   = new quadlineart;
  Qqt   =  new quadquadrilatt;
  Lqat  = new quadlinaxisym;
  Ltett = new lintett;
  Lht   = new linhext;
  Qht   = new quadhext;



  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "end");
  in->kwdmode = ignore;
  // main input of properties
  err = input(in, d);
  Tt->alloc_prep(Top->nodes, Top->elements);
  if(err)
  {
    xfclose(in);
    return(err);
  }
  // reading output description
  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "begin");
  in->kwdmode = ignore;

  Outdt->read(in);
  xfclose(in);

  FILE *out = fopen(argv[2], "wt");
  if (out==NULL){
    fprintf (stderr,"\n Output file couldn't be opened.");
    fprintf (stderr,"\n Try it again!\n\n");
    return(7);
  }
  fprintf(stdout, "\nTopology export . . .");
  Top->exporttop(Gtt);
  fprintf(stdout, " O.K.\n");
  fprintf(stdout, "\nProblem description output . . .");
  Tp->print(out);
  fprintf(stdout, " O.K.\n");
  output(out, d);
  fprintf(stdout, "\nOutdriver description output . . .\n");
  Outdt->print(out);
  fclose(out);

  fprintf(stderr, "\n");

  delete d;
  delete Tp;  
  delete Tt;  
  delete Gtt;  
  delete Tm;  
  delete Tc;  
  delete Tb;
  delete Outdt;
  return(0);
}
