#include "elemswitcht.h"
#include "globalt.h"
#include "globmatt.h"

/**
   function computes conductivity %matrix of one element
   
   @param eid - element id
   @param lcid - load case id
   @param km - conductivity %matrix of one element

*/
void conductmat (long eid,long lcid,matrix &km)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);
  
  switch (te){
  case barlint:{
    Lbt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case barlintax:{
    Lbat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case barquadt:{
    Qbt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case barquadtax:{
    Qbat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case trlint:{
    Ltt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case trlaxisym:{
    Ltat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadlint:{
    Lqt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadquadt:{
    Qqt->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadquadtax:{
    Qqat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadlaxisym:{
    Lqat->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case lineartett:{
    Ltett->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case linearhext:{
    Lht->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  case quadratichext:{
    Qht->res_conductivity_matrix (eid,lcid,km);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  
}

/**
   function computes capacity %matrix of one element

   @param eid - element id
   @param lcid - load case id
   @param km - capacity %matrix of one element

*/
void capacmat (long eid,long lcid,matrix &cm)
{
  elemtypet te;

  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->res_capacity_matrix (eid,cm);
    break;
  }
  case barlintax:{
    Lbat->res_capacity_matrix (eid,cm);
    break;
  }
  case barquadt:{
    Qbt->res_capacity_matrix (eid,cm);
    break;
  }
  case barquadtax:{
    Qbat->res_capacity_matrix (eid,cm);
    break;
  }
  case trlint:{
    Ltt->res_capacity_matrix (eid,cm);
    break;
  }
  case trlaxisym:{
    Ltat->res_capacity_matrix (eid,cm);
    break;
  }
  case quadlint:{
    Lqt->res_capacity_matrix (eid,cm);
    break;
  }
  case quadquadt:{
    Qqt->res_capacity_matrix (eid,cm);
    break;
  }
  case quadquadtax:{
    Qqat->res_capacity_matrix (eid,cm);
    break;
  }
  case quadlaxisym:{
    Lqat->res_capacity_matrix (eid,cm);
    break;
  }
  case lineartett:{
    Ltett->res_capacity_matrix (eid,cm);
    break;
  }
  case linearhext:{
    Lht->res_capacity_matrix (eid,cm);
    break;
  }
  case quadratichext:{
    Qht->res_capacity_matrix (eid,cm);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
}



/**
   function approximates nodal values to integration points
   
   @param eid - element id
   
   JK
*/
void intpointvalues (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->intpointval (eid);
    break;
  }
  case barlintax:{
    Lbat->intpointval (eid);
    break;
  }
  case barquadt:{
    Qbt->intpointval (eid);
    break;
  }
  case barquadtax:{
    Qbat->intpointval (eid);
    break;
  }
  case trlint:{
    Ltt->intpointval (eid);
    break;
  }
  case trlaxisym:{
    Ltat->intpointval (eid);
    break;
  }
  case quadlint:{
    Lqt->intpointval (eid);
    break;
  }
  case quadquadt:{
    Qqt->intpointval (eid);
    break;
  }
  case quadquadtax:{
    Qqat->intpointval (eid);
    break;
  }
  case quadlaxisym:{
    Lqat->intpointval (eid);
    break;
  }
  case lineartett:{
    Ltett->intpointval (eid);
    break;
  }
  case linearhext:{
    Lht->intpointval (eid);
    break;
  }
  case quadratichext:{
    Qht->intpointval (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function computes gradients on integration points
   
   @param eid - element id
   
   JK, 10.7.2008
*/
void intpointgradients (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->intpointgrad (eid);
    break;
  }
  case barlintax:{
    Lbat->intpointgrad (eid);
    break;
  }
  case barquadt:{
    Qbt->intpointgrad (eid);
    break;
  }
  case barquadtax:{
    Qbat->intpointgrad (eid);
    break;
  }
  case trlint:{
    Ltt->intpointgrad (eid);
    break;
  }
  case trlaxisym:{
    Ltat->intpointgrad (eid);
    break;
  }
  case quadlint:{
    Lqt->intpointgrad (eid);
    break;
  }
  case quadquadt:{
    Qqt->intpointgrad (eid);
    break;
  }
  case quadquadtax:{
    Qqat->intpointgrad (eid);
    break;
  }
  case quadlaxisym:{
    Lqat->intpointgrad (eid);
    break;
  }
  case lineartett:{
    Ltett->intpointgrad (eid);
    break;
  }
  case linearhext:{
    Lht->intpointgrad (eid);
    break;
  }
  case quadratichext:{
    Qht->intpointgrad (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function approximates nodal values of array other to integration points
   
   @param eid - element id
   
   JK
*/
void intpointother (long eid)
{
  elemtypet te;
  
  te=Tt->give_elem_type (eid);

  switch (te){
  case barlint:{
    Lbt->intpointother (eid);
    break;
  }
  case barlintax:{
    Lbat->intpointother (eid);
    break;
  }
  case barquadt:{
    Qbt->intpointother (eid);
    break;
  }
  case barquadtax:{
    Qbat->intpointother (eid);
    break;
  }
  case trlint:{
    Ltt->intpointother (eid);
    break;
  }
  case trlaxisym:{
    Ltat->intpointother (eid);
    break;
  }
  case quadlint:{
    Lqt->intpointother (eid);
    break;
  }
  case quadquadt:{
    Qqt->intpointother (eid);
    break;
  }
  case quadquadtax:{
    Qqat->intpointother (eid);
    break;
  }
  case quadlaxisym:{
    Lqat->intpointother (eid);
    break;
  }
  case lineartett:{
    Ltett->intpointother (eid);
    break;
  }
  case linearhext:{
    Lht->intpointother (eid);
    break;
  }
  case quadratichext:{
    Qht->intpointother (eid);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function computes contributions from source of quantity
   
   @param lcid - load case id
   @param eid - element id
   @param nodval - array of source nodal values
   @param lv - local %vector
   
   JK, 25.6.2005
*/
void source_vector (long lcid,long eid,vector &nodval,vector &lv)
{
  elemtypet te;
  
  //  type of element
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case barlintax:{
    Lbat->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case trlint:{
    Ltt->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case trlaxisym:{
    Ltat->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadlint:{
    Lqt->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadlaxisym:{
    Lqat->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadquadt:{
    Qqt->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadquadtax:{
    Qqat->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case lineartett:{
    Ltett->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case linearhext:{
    Lht->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  case quadratichext:{
    Qht->res_quantity_source_vector (lv,nodval,lcid,eid);
    break;
  }
  default:{
    print_err("unknown element type is required", __FILE__, __LINE__, __func__);
  }
  }
  
}

/**
   function computes other values in nodes
   
   @param lcid - load case id
*/
void compute_nodeotherst (long lcid)
{
  long i;
  elemtypet te;
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].nullother();
  }
  
  for (i=0;i<Tt->ne;i++){
    if (Gtt->leso[i]==1){
      
      te = Tt->give_elem_type (i);
      
      switch (te){
      case barlint:{
	Lbt->nod_others (lcid,i,0,0);
	break;
      }
      case barlintax:{
	Lbat->nod_others (lcid,i,0,0);
	break;
      }
      case barquadt:{
	Qbt->nod_others (lcid,i,0,0);
	break;
      }
      case barquadtax:{
	Qbat->nod_others (lcid,i,0,0);
	break;
      }
      case quadlint:{
	Lqt->nod_others (lcid,i,0,0);
	break;
      }
      case trlint:{
	Ltt->nod_others (lcid,i,0,0);
	break;
      }
      case trlaxisym:{
	Ltat->nod_others (lcid,i,0,0);
	break;
      }
      case quadquadt:{
	Qqt->nod_others (lcid,i,0,0);
	break;
      }
      case quadquadtax:{
	Qqat->nod_others (lcid,i,0,0);
	break;
      }
      case quadlaxisym:{
	Lqat->nod_others (lcid,i,0,0);
	break;
      }
      case lineartett:{
	Ltett->nod_others (lcid,i,0,0);
	break;
      }
      case linearhext:{
	Lht->nod_others (lcid,i,0,0);
	break;
      }
      case quadratichext:{
	Qht->nod_others (lcid,i,0,0);
	break;
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
      }
      }
    }
  }
  
  for (i=0;i<Tt->nn;i++){
    Tt->nodes[i].other_averageval ();
  }
}

/**
   function assembles %vector of nodal internal fluxes

   @param lcid - load case id
   @param intflux - array containing internal nodal fluxes
   
*/
void internal_fluxes (double *intflux,long n)
{
  long i,ne,ndofe,*cn;
  elemtypet te;
  vector lif;
  
  ne = Tt->ne;
  nullv (intflux,n);

  for (i=0;i<ne;i++){
    if (Gtt->leso[i]==1){

      te = Tt->give_elem_type (i);
      ndofe = Gtt->give_ndofe (i);
      allocv (ndofe,lif);
      fillv (0.0,lif);
      
      cn = new long [ndofe];
      Gtt->give_code_numbers (i,cn);
      
      switch (te){
      case barlint:{
	Lbt->res_internal_fluxes (i,lif);
	break;
      }
      case barlintax:{
	Lbat->res_internal_fluxes (i,lif);
	break;
      }
      case barquadt:{
	Qbt->res_internal_fluxes (i,lif);
	break;
      }
      case barquadtax:{
	Qbat->res_internal_fluxes (i,lif);
	break;
      }
      case trlint:{
	Ltt->res_internal_fluxes (i,lif);
	break;
      }
      case trlaxisym:{
	Ltat->res_internal_fluxes (i,lif);
	break;
      }
      case quadlint:{
	Lqt->res_internal_fluxes (i,lif);
	break;
      }
      case quadquadt:{
	Qqt->res_internal_fluxes (i,lif);
	break;
      }
      case quadquadtax:{
	Qqat->res_internal_fluxes (i,lif);
	break;
      }
      case quadlaxisym:{
	Lqat->res_internal_fluxes (i,lif);
	break;
      }
      case lineartett:{
	Ltett->res_internal_fluxes (i,lif);
	break;
      }
      case linearhext:{
	Lht->res_internal_fluxes (i,lif);
	break;
      }
      case quadratichext:{
	Qht->res_internal_fluxes (i,lif);
	break;
      }
      default:{
	print_err("unknown element type is required",__FILE__,__LINE__,__func__);
	abort ();
      }
      }
      
      locglob (intflux,lif.a,cn,ndofe);
      destrv (lif);
      delete [] cn;
    }
  }

  trfel_bound_flux (0,intflux,n);

}

/**
   function computes nodal values on element defined by
   prescribed fluxes (Neumann boundary condition)
   
   @param lv - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   @param i - index in the list of loaded elements
   
   JK, 22.11.2008
*/
void elem_neumann_vector (vector &lv,long lcid,long eid,long i)
{
  elemtypet te;
  //  element type
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case barlintax:{
    Lbat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case barquadt:{
    Qbt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case barquadtax:{
    Qbat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case trlint:{
    Ltt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case trlaxisym:{
    Ltat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadlint:{
    Lqt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadlaxisym:{
    Lqat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadquadt:{
    Qqt->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadquadtax:{
    Qqat->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case lineartett:{
    Ltett->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case linearhext:{
    Lht->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  case quadratichext:{
    Qht->res_convection_vector (lv,lcid,eid,i);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}


/**
   function computes nodal values on element defined by
   prescribed transmission (Newton boundary condition)
   
   this function computes \int \kappa T_{ext}
   
   @param lv - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   @param i - index in the list of loaded elements
   
   JK, 22.11.2008
*/
void elem_newton_vector (vector &lv,long lcid,long eid,long i)
{
  elemtypet te;
  //  element type
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case barlintax:{
    Lbat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case barquadt:{
    Qbt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case barquadtax:{
    Qbat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case trlint:{
    Ltt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case trlaxisym:{
    Ltat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadlint:{
    Lqt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadlaxisym:{
    Lqat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadquadt:{
    Qqt->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadquadtax:{
    Qqat->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case lineartett:{
    Ltett->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case linearhext:{
    Lht->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  case quadratichext:{
    Qht->res_transmission_vector (lv,lcid,eid,i);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function computes nodal values of boundary fluxes on element
   caused by transmission

   this function differs from the function elem_newton_vector
   this function computes fluxes on boundaries while the
   function elem_newton_vector computes terms for the right hand
   side %vector
   
   this function computes \int \kappa (T-T_{ext})

   @param lv - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   @param i - index in the list of loaded elements
   
   JK, 22.11.2008
*/
void elem_transmission_flux (vector &lv,long lcid,long eid,long i)
{
  elemtypet te;
  //  element type
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case barlintax:{
    Lbat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case barquadt:{
    Qbt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case barquadtax:{
    Qbat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case trlint:{
    Ltt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case trlaxisym:{
    Ltat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadlint:{
    Lqt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadlaxisym:{
    Lqat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadquadt:{
    Qqt->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadquadtax:{
    Qqat->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case lineartett:{
    Ltett->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case linearhext:{
    Lht->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  case quadratichext:{
    Qht->res_boundary_flux (lv,lcid,eid,i);
    break;
  }
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}

/**
   function computes 
   
   @param lv - %vector of nodal values
   @param lcid - load case id
   @param eid - element id
   
   JK, 22.11.2008
*/
void volume_rhs_vector (vector &lv,long lcid,long eid)
{
  elemtypet te;
  //  element type
  te=Tt->elements[eid].te;
  
  switch (te){
  case barlint:{
    Lbt->res_volume_rhs_vector (lv,eid,lcid);
    break;
  }
    /*
      case barlintax:{
      break;
      }
      case barquadt:{
      break;
      }
      case barquadtax:{
      break;
      }
      case trlint:{
      break;
      }
      case trlaxisym:{
      break;
      }
    */
  case quadlint:{
    break;
  }
    /*
      case quadlaxisym:{
      break;
      }
      case quadquadt:{
      break;
      }
      case quadquadtax:{
      break;
      }
      case lineartett:{
      break;
      }
      case linearhext:{
      break;
      }
      case quadratichext:{
      break;
      }
    */
  default:{
    print_err("unknown element type is required",__FILE__,__LINE__,__func__);
  }
  }
  
}
