#include "intpointst.h"
#include "globalt.h"
#include <string.h>

intpointst::intpointst ()
{
  //  number of components in gradients
  ncompgrad=0;
  //  number of components in array other
  ncompother=0;
  //  number of components in array eqother
  ncompeqother=0;
  
  //  type of material
  tm = (mattypet) 0;
  //  material id
  idm=0;
  
  //  array of actual values
  av=NULL;
  //  array of previous values (values from previous time step)
  pv=NULL;
  //  array of gradients
  grad=NULL;
  //  array of fluxes
  fluxes=NULL;
  //  auxiliary array other
  other = NULL;
  //  auxiliary array eqother
  eqother = NULL;
}

intpointst::~intpointst (void)
{
  long i;
  delete [] av;  
  delete [] pv;
  for (i=0; i<Tp->ntm; i++)
  {
    if (grad != NULL)
      delete [] grad[i];
    if (fluxes != NULL)
      delete [] fluxes[i];
  }
  delete [] grad;  delete [] fluxes;
  delete [] other;   
  delete [] eqother;
}

/**
   function reads data connected with integration point
   
   @param in - input file
*/
void intpointst::read (FILE *in)
{
  fscanf (in,"%d %ld",(int*)&tm,&idm);
  idm--;
}


/**
   function allocates memory for actual values, previous values, axiliary values,
   equilibriated auxiliary values, gradients and fluxes
   
   @param ipp - integration point id
*/
void intpointst::alloc (long ipp)
{
  long i,ntm;

  //  number of transported media
  ntm = Tp->ntm;
  //  number of components of gradients
  ncompgrad = Tp->gdim;
  //  number of components of array other
  ncompother = Tm->givencompother();
  //  number of components of array eqother
  ncompeqother = Tm->givencompeqother(ipp,0);

  if (ncompother > 0){
    other = new double[ncompother];
  }
  
  if (ncompeqother > 0){
    eqother = new double[ncompeqother];
  }
  
  
  //  array for actual values
  av = new double [ntm];
  //  array for previous values (values from the previous time step)
  pv = new double [ntm];
  //  array for gradients
  grad = new double* [ntm];
  //  array for fluxes
  fluxes = new double* [ntm];
  
  for (i=0;i<ntm;i++){
    grad[i] = new double [ncompgrad];
    fluxes[i] = new double [ncompgrad];
  }
  
  clean ();
}


/**
   function cleans all arrays defined on integration point
   
   JK
*/
void intpointst::clean ()
{
  long i,j,ntm;
  
  //  number of transported media
  ntm = Tp->ntm;
  
  for (i=0;i<ntm;i++){
    av[i]=0.0;
    pv[i]=0.0;
    for (j=0;j<ncompgrad;j++){
      grad[i][j]=0.0;
      fluxes[i][j]=0.0;
    }
  }
  
  for (i=0;i<ncompother;i++){
    other[i]=0.0;
  }
  
  for (i=0;i<ncompeqother;i++){
    eqother[i]=0.0;
  }
}

/**
   function saves all arrays into the auxiliary file
   
   @param aux - pointer to auxiliary file
   
   JK, 21.9.2004
*/
void intpointst::save_data (FILE *aux)
{
  long i,j,ntm,gdim;
  
  //  number of transported media
  ntm = Tp->ntm;
  //  number of geometrical dimesnions
  gdim = Tp->gdim;

  for (i=0;i<ntm;i++){
    fprintf (aux,"%e\n",av[i]);
  }
  for (i=0;i<ntm;i++){
    fprintf (aux,"%e\n",pv[i]);
  }
  for (i=0;i<ntm;i++){
    for (j=0;j<gdim;j++){
      fprintf (aux,"%e\n",grad[i][j]);
    }
  }
  for (i=0;i<ntm;i++){
    for (j=0;j<gdim;j++){
      fprintf (aux,"%e\n",fluxes[i][j]);
    }
  }
  
}

/**
   function restores all arrays from the auxiliary file
   
   @param aux - pointer to auxiliary file
   
   JK, 21.9.2004
*/
void intpointst::restore_data (FILE *aux)
{
  long i,j,ntm,gdim;
  
  //  number of transported media
  ntm = Tp->ntm;
  //  number of geometrical dimesnions
  gdim = Tp->gdim;

  for (i=0;i<ntm;i++){
    fscanf (aux,"%le\n",&av[i]);
  }
  for (i=0;i<ntm;i++){
    fscanf (aux,"%le\n",&pv[i]);
  }
  for (i=0;i<ntm;i++){
    for (j=0;j<gdim;j++){
      fscanf (aux,"%le\n",&grad[i][j]);
    }
  }
  for (i=0;i<ntm;i++){
    for (j=0;j<gdim;j++){
      fscanf (aux,"%le\n",&fluxes[i][j]);
    }
  }
  
}

/**
   function moves array of actual values to array of previous values
   
   JK, 29.5.2007
*/
void intpointst::actual_previous_change ()
{
  long i,ntm;
  
  //  number of transported media
  ntm = Tp->ntm;
  
  for (i=0;i<ntm;i++){
    pv[i]=av[i];
  }
}
