/*
    File:             onemedium.cpp
    Author:           Tomas Krejci,  20.12.2002
    Purpose:          computes conductivity and capacity matrices in a material point for one medium trasport
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "aliast.h"
#include "onemedium.h"
#include "globalt.h"

med1::med1()
{  
  //not completed
  scale = 1.0;
}
med1::~med1()
{}


/**
   function computes conductivity %matrix D in a material point for one medium transfer
   
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point
   
*/
void med1::matcond (matrix &d,long ri,long ci,long ipp)
{
  long i;
  long n;
  n = d.n;
  
  i = Tm->ip[ipp].idm;
  
  switch (Tp->mednam){//names of transported media
  case moisture:{
    double k;
    switch (Tm->ip[ipp].tm){
    case bazantpedersen:{//material type
      double w,phi;
      w = Tm->ip[ipp].av[0];
      phi = Tm->bazped[i].inverse_sorption_isotherm(w);
      k = Tm->bazped[i].get_delta_gw(phi,w);
      break;      
    }
    case pedersen:{
      double w,phi;
      w = Tm->ip[ipp].av[0];
      phi = Tm->ped[i].inverse_sorption_isotherm(w);
      k = Tm->ped[i].get_delta_gw(phi,w);
      break;
    }

    case sejtkr:{
      double pw;
      
      pw = Tm->ip[ipp].av[0];
      k = Tm->sejtkrm[i].get_kww(pw);
      break;
    }

    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }  
    
    switch (n){
    case 1:{
      fillm(0.0,d);
      d[0][0] = k;//1D
      break;
    }
    case 2:{
      fillm(0.0,d);
      d[0][0] = k;   d[0][1] = 0.0;
      d[1][0] = 0.0; d[1][1] = k;//2D
      break;
    }
    case 3:{
      fillm(0.0,d);
      d[0][0]=k;   d[0][1]=0.0; d[0][2]=0.0;
      d[1][0]=0.0; d[1][1]=k;   d[1][2]=0.0;
      d[2][0]=0.0; d[2][1]=0.0; d[2][2]=k;//3D
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
    
  case heat:{
    switch (Tm->ip[ipp].tm){//material type
    case isotransmat:{
      Tm->itrm[i].matcond(d,ri,ci,ipp);
      break;
    }
    case nlisotransmat:{
      Tm->nlitrm[i].matcond(d,ri,ci,ipp);
      break;
    }
    case discontisotrmat:{
      Tm->ditrm[i].matcond(d,ri,ci,ipp);
      break;
    }
    case cernyconcrete:{
      Tm->cernym[i].matcond(d,ri,ci,ipp);
      break;
    }
    case cementhydrmat:{
      Tm->cemhydr[i].matcond(d,ri,ci,ipp);
      break;
    }
    case lincoupledmat:{
      Tm->lcmat[i].matcond(d,ri,ci,ipp);
      break;
    }
    default:{
      print_err("\n unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }    
}


/**
   function computes capacity %matrix C in a material point for one medium transfer

   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point
*/
void med1::matcap (double &c,long ri,long ci,long ipp)
{
  long i;
  
  i = Tm->ip[ipp].idm;
  
  switch (Tp->mednam){//names of transported media
  case moisture:{
    
    switch (Tm->ip[ipp].tm){//material type
    case bazantpedersen:{
      c = 1.0;
      
      break;
    }
    case pedersen:{
      c = 1.0;
      
      break;
    }
      
    case sejtkr:{
      double pw;
      
      pw = Tm->ip[ipp].av[0];
      c = Tm->sejtkrm[i].get_capww(pw);
      break;
    }

    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }  
    break;
  }
  case heat:{
    
    switch (Tm->ip[ipp].tm){
    case isotransmat:{//material type
      Tm->itrm[i].matcap(c,ri,ci,ipp);
      break;
    }
    case nlisotransmat:{//material type
      Tm->nlitrm[i].matcap(c,ri,ci,ipp);
      break;
    }
    case discontisotrmat:{//material type
      Tm->ditrm[i].matcap(c,ri,ci,ipp);
      break;
    }
    case cernyconcrete:{
      Tm->cernym[i].matcap(c,ri,ci,ipp);
      break;
    }
    case cementhydrmat:{
      Tm->cemhydr[i].matcap(c,ri,ci,ipp);
      break;
    }
    case lincoupledmat:{
      Tm->lcmat[i].matcap(c,ri,ci,ipp);
      break;
    }
    default:{
      print_err("\n unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}

/**
   function computes new transmission coefficient for transmission on the boundary
   (third kind of boundary condition)

   @param trc     - prescribed transmission coefficient on the boundary
   @param ri      - row index
   @param ci      - column index
   @param nn      - node id
   @param bc      - type of boundary condition
   @param ipp     - number of first integration point on element
*/
double med1::transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp)
{
  double new_trc;

  switch (Tp->mednam){//names of transported media
  case moisture:{
    
    switch (bc){//type of prescribed variable
    case 30:{//mass transmission
      new_trc=1.0;
      break;
    }
    default:{
      print_err("\n No real boundary condition is prescribed in ", __FILE__, __LINE__, __func__);
      exit(0);
    }
    }
  
    break;
  }
  case heat:{

    switch (bc){//type of prescribed variable
    case 11:{
      new_trc=trc;
    }
    case 30:{//heat transmission
      new_trc=1.0;
      break;
    }
    case 31:{//heat transmission for testing
      new_trc=0.0;
      break;
    }
    case 90:{//radiation
      new_trc=0.0;
      break;
    }
    default:{
      print_err("\n No real boundary condition is prescribed in ", __FILE__, __LINE__, __func__);
      exit(0);
    }
    }
    
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }    
  new_trc = new_trc*trc;
  return(new_trc);
}



/**
   function computes new nodal value for transmission on the boundary
   (third kind of boundary condition)

   @param nodval     - nodal value defined on boundary
   @param trc2       - second prescribed transmission coefficient on the boundary, 
                       if it is needed (for example heat radiation coef.)
   @param ri         - row index
   @param ci         - column index
   @param nn         - node id
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element
*/
double med1::transmission_nodval(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp)
{
  double new_nodval;
  long k;
  double t,w;
  
  switch (Tp->mednam){//names of transported media
  case moisture:{
    
    k=Gtt->give_dof(nn,0);
    if (k>0)   {w = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
    if (k==0)  {w = 0.0;}
    if (k<0)   {w = Tb->lc[0].pv[0-k-1].getval();}

    switch (bc){//type of prescribed variable
    case 30:{//mass transmission
      new_nodval = nodval;
      break;
    }
    default:{
      print_err("\n No real boundary condition is prescribed in ", __FILE__, __LINE__, __func__);
      exit(0);
    }
    }
     
    break;
  }
  case heat:{

    k=Gtt->give_dof(nn,0);
    if (k>0)   {t = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
    if (k==0)  {t = 0.0;}
    if (k<0)   {t = Tb->lc[0].pv[0-k-1].getval();}

    switch (bc){//type of prescribed variable
    case 11:{
      new_nodval = nodval;
      break;
    }
    case 30:{//heat transmission
      new_nodval = nodval;
      break;
    }
    case 31:{//heat transmission for testing (and for boundary flux)
      new_nodval = (nodval - t);
      break;
    }
    case 90:{//radiation
      new_nodval = (nodval - t) + trc2*(nodval*nodval*nodval*nodval - t*t*t*t);//inverse eq. (trc2 = e_sigma0/trc)
      break;
    }
    default:{
      print_err("\n No real boundary condition is prescribed in ", __FILE__, __LINE__, __func__);
      exit(0);
    }
    }
    
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }    
  return(new_nodval);
}



/**
   function computes flux on the boundary for transmission on the boundary
   (third kind of boundary condition)

   @param nodval  - prescribed nodal value on boundary
   @param trc2    - second prescribed transmission coefficient on the boundary, 
                    if it is needed (for example heat radiation coef.)
   @param ri      - row index
   @param ci      - column index
   @param nn      - node id
   @param bc      - type of boundary condition
   @param ipp     - number of first integration point on element
*/
double med1::transmission_flux(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp)
{
  double flux;
  long k;
  double t,w;
  
  switch (Tp->mednam){//names of transported media
  case moisture:{
    
    k=Gtt->give_dof(nn,0);
    if (k>0)   {w = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
    if (k==0)  {w = 0.0;}
    if (k<0)   {w = Tb->lc[0].pv[0-k-1].getval();}
    
    switch (bc){//type of prescribed variable
    case 30:{//mass transmission
      flux = (nodval - w);
      break;
    }
    default:{
      print_err("\n No real boundary condition is prescribed in ", __FILE__, __LINE__, __func__);
      exit(0);
    }
    }
    
    break;
  }
  case heat:{

    k=Gtt->give_dof(nn,0);
    if (k>0)   {t = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
    if (k==0)  {t = 0.0;}
    if (k<0)   {t = Tb->lc[0].pv[0-k-1].getval();}

    switch (bc){//type of prescribed variable
    case 30:{//heat transmission
      flux = (nodval - t);
      break;
    }
    case 31:{//heat transmission for testing (and for boundary flux)
      flux = (nodval - t);
      break;
    }
    case 90:{//radiation
      flux = (nodval - t) + trc2*(nodval*nodval*nodval*nodval - t*t*t*t);//inverse eq. (trc2 = e_sigma0/trc)
      break;
    }
    default:{
      print_err("\n No real boundary condition is prescribed in ", __FILE__, __LINE__, __func__);
      exit(0);
    }
    }
    
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }    

  return(flux);

}



/**
   function computes all variables in nodes
   @param compother - number of other components
   @param ipp - first integration point on element
   @param r - %vector of unknowns on actual node
*/

double med1::compute_othervalues (long compother,long ipp,double *r)
{
  long i;
  double other;

  switch (Tp->mednam){//names of transported media
  case moisture:{
    i = Tm->ip[ipp].idm;
    
    switch (Tm->ip[ipp].tm){//material type
    case bazantpedersen:{
      double w,t;
      
      w = r[0];
      t = 0.0;
      
      other = Tm->bazped[i].get_othervalue(compother,w,t);
      break;
    }
    case pedersen:{
      double w,t;
      
      w = r[0];
      t = 0.0;

      other = Tm->ped[i].get_othervalue(compother,w,t);
      break;
    }
    default:{
    print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
  return(other);
}


/**
   function prints names of all variables in nodes
   @param out - output file
   @param compother - number of other components
   @param ipp - first integration point on element
*/

void med1::print_othervaluesnames (FILE *out,long ipp,long compother)
{
  long i;
  
  switch (Tp->mednam){//names of transported media
  case moisture:{
    i = Tm->ip[ipp].idm;
    
    switch (Tm->ip[ipp].tm){//material type
    case bazantpedersen:{
      Tm->bazped[i].print_othervalue_name(out,compother);
      break;
    }
    case pedersen:{
      Tm->ped[i].print_othervalue_name(out,compother);
      break;
    }
    default:{
    print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}

