#ifndef PROBDESCT_H
#define PROBDESCT_H

#include <stdio.h>
#include "aliast.h"
#include "galias.h"
#include "gfunct.h"
#include "timecontr.h"
#include "slesolv.h"
#include "nonlinmant.h"
#include <iotools.h>
#include "xfile.h"

/**
   class probdesc
   
   it is one of the 5 most important classes of the program
   (probdesc, transtop, transmat, transbclc, transcrsec)
   
   class contains general information about solved problem
*/

class probdesct
{
 public:
  probdesct (void);
  ~probdesct (void);
  void read (XFILE *in);
  void print (FILE *out);
  void filename_decompositiont (const char *file,char *&patht,char *&namet,char *&suffixt);
  
  

  char *path;
  char *filename;
  char *suffix;
  
  /// keyword processing option
  pkwd_swt kwdswt;

  ///  problem name
  char name[1001];

  ///  type of problem
  problemtypet tprob;
  
  ///  type of transported matter
  transmattert tmatt;
  ///  number of transported matters=number of row(column) blocks in matrices
  long ntm;
  ///  names of transported media
  mednamest mednam;
  /// scale of transported media
  double scale1,scale2,scale3,scale4;

  ///  data about solver of system of nonlinear equations
  nonlinmant nlman;

  ///  gravity acceleration is taken into account
  gravityaccelerationt tgravity;
  double gr1,gr2,gr3;

  ///  geometrical dimension
  long gdim;
  ///  savemode - deals with the number of integration points
  ///  savemode=0 - greater number of integration points
  ///  savemode=1 - smaller number of integration points
  long savemode;
  
  ///  type of transport solver
  ///  trsolv = 1 - full newton method (matrices are recomputed in all internal loops)
  ///  trsolv = 2 - modified Newton method (matrices are recomputed only in new increment)
  transpsolver trsolv;

  ///  type of residuum computation
  ///  trestype = 1 - residuum is computed from internal fluxes
  ///  trestype = 2 - residuum is computed as difference between right and left hand side
  transpresiduumtype trestype;
  
  ///  convergence control for full newton-raphson
  ///  yes = 1
  ///  no  = 0 
  transconvergcontroltype convergcontrolt;

  //  STORAGE TYPE
  ///  type of storage of conductivity matrix
  storagetype tstorkm;
  ///  type of storage of capacity matrix
  storagetype tstorcm;

  /// diagonalization of capacity matrix
  long diagcap;
  
  ///  data about solver of system of linear equations
  slesolv *ssle;

  ///  time controller
  timecontr timecont;
  ///  time
  double time;
  
  ///  threshold for rejection from compressed storages
  double limit;
  ///  computer zero
  double zero;
  ///  threshold for the size of the right hand side %vector
  double *threshrhs;

  /***************************/
  /*  non-stationary solver  */
  /***************************/

  ///  coefficient alpha
  ///  alpha=0.0 - explicit algorithm
  ///  alpha=1/2 - Crank-Nicholson scheme
  ///  alpha=1.0 - implicit algorithm
  double alpha;
    
  ///  required error in Newton-Raphson method
  double err;
  double *errarr;
  ///  maximum number of inner iterations in Newton-Rapson method
  long nii;

  ///  stochastic calculation indicator
  long stochasticcalc;
  
  
  ///  compute gradients
  ///  gradcomp=0 - gradients are not computed
  ///  gradcomp=1 - gradients are computed
  long gradcomp;
  ///  compute fluxes
  ///  fluxcomp=0 - fluxes are not computed
  ///  fluxcomp=1 - fluxes are computed
  long fluxcomp;
  ///  compute other values
  ///  othercomp=0 - values from array other are not computed
  ///  othercomp=1 - values from array other are computed
  long othercomp;
  ///  compute eq_other values 
  ///  eqothercomp=0 - values from array eqother are not computed
  ///  eqothercomp=1 - values from array eqother are computed
  long eqothercomp;
  
  ///  gradient averaging (if gradaver=1 then average value)
  long gradaver;
  ///  flux averaging (if fluxaver=1 then average value)
  long fluxaver;
  ///  other values averaging
  long otheraver;
  ///  eq_other values averaging
  long eqotheraver;
  
  ///  storage of actual nodal values on nodes
  ///  nvs=0 - actual nodal values are not stored
  ///  nvs=1 - actual nodal values are stored
  long nvs;
  ///  storage of initial nodal values on nodes
  ///  invs=0 - initial nodal values are not stored
  ///  invs=1 - initial nodal values are stored
  long invs;
  ///  storage of nodal values from previous time step on nodes
  ///  pnvs=0 - previous nodal values are not stored
  ///  pnvs=1 - previous nodal values are stored
  long pnvs;
  ///  storage of time derivative of  nodal values on nodes
  ///  tdnvs=0 - time derivative of nodal values are not stored
  ///  tdnvs=1 - time derivative of nodal values are stored
  long tdnvs;
  
  ///  type of time printing
  timetypeprint tprt;
  

  ///  number of simulation
  long ns;

};

#endif
