#ifndef saltmat4_H
#define saltmat4_H

#include <stdio.h>
#include "genfile.h"

/**
   class describes material model which deals with simultaneous
   transport of moisture, salt and salt crystals
   
   model uses array eqother
   eqother[0] - moisture diffusivity (kappa)
   eqother[1] - binding izoterm
   eqother[2] - salt diffusivity (D)
   eqother[3] - ???
   
   JM, 29.5.2007
*/
class saltmat4
{
 public:
  saltmat4 (void);    //constructor
  ~saltmat4 (void);   //destructor
  
  void matcond (matrix &d,long ri,long ci,long ipp);
  void matcond2 (matrix &d,long ri,long ci,long ipp);
  void matcap (double &c,long ri,long ci,long ipp);
  
  void matcond1d (matrix &d,long ri,long ci,long ipp);
  void matcond2d (matrix &d,long ri,long ci,long ipp);
  void matcond3d (matrix &d,long ri,long ci,long ipp);
  void matcond4d (matrix &d,long ri,long ci,long ipp);
  void matcond2d2 (matrix &d,long ri,long ci,long ipp);

  void read (XFILE *in);

  double k11 (double x1,double x2,double x3,double x4,long ipp);
  double k12 (double x1,double x2,double x3,double x4,long ipp);
  double k13 (double x1,double x2,double x3,double x4,long ipp);
  double k14 (double x1,double x2,double x3,double x4,long ipp);

  double k21 (double x1,double x2,double x3,double x4,long ipp);
  double k22 (double x1,double x2,double x3,double x4,long ipp);
  double k23 (double x1,double x2,double x3,double x4,long ipp);
  double k24 (double x1,double x2,double x3,double x4,long ipp);

  double k31 (double x1,double x2,double x3,double x4,long ipp);
  double k32 (double x1,double x2,double x3,double x4,long ipp);
  double k33 (double x1,double x2,double x3,double x4,long ipp);
  double k34 (double x1,double x2,double x3,double x4,long ipp);

  double k41 (double x1,double x2,double x3,double x4,long ipp);
  double k42 (double x1,double x2,double x3,double x4,long ipp);
  double k43 (double x1,double x2,double x3,double x4,long ipp);
  double k44 (double x1,double x2,double x3,double x4,long ipp);

  double c11 (double x1,double x2,double x3,double x4,long ipp);
  double c12 (double x1,double x2,double x3,double x4,long ipp);
  double c13 (double x1,double x2,double x3,double x4,long ipp);
  double c14 (double x1,double x2,double x3,double x4,long ipp);

  double c21 (double x1,double x2,double x3,double x4,long ipp);
  double c22 (double x1,double x2,double x3,double x4,long ipp);
  double c23 (double x1,double x2,double x3,double x4,long ipp);
  double c24 (double x1,double x2,double x3,double x4,long ipp);

  double c31 (double x1,double x2,double x3,double x4,long ipp);
  double c32 (double x1,double x2,double x3,double x4,long ipp);
  double c33 (double x1,double x2,double x3,double x4,long ipp);
  double c34 (double x1,double x2,double x3,double x4,long ipp);

  double c41 (double x1,double x2,double x3,double x4,long ipp);
  double c42 (double x1,double x2,double x3,double x4,long ipp);
  double c43 (double x1,double x2,double x3,double x4,long ipp);
  double c44 (double x1,double x2,double x3,double x4,long ipp);

  void auxiliarydata (double x1,double x2,double x3,double x4);
  
  double transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp);
  double transmission_nodval(double nodval,long ri,long ci,long nn,long bc,long ipp);
  double transmission_flux(double nodval,long ri,long ci,long nn,long bc,long ipp);

  double get_transmission_nodval_11(double bv,double x1,double x2,double x3,double x4,long bc,long ipp);
  double get_transmission_transcoeff_11(double x1,double x2,double x3,double x4,long bc,long ipp);
  double get_transmission_flux_11(double bv,double x1,double x2,double x3,double x4,long bc,long ipp);

  
  double get_othervalue(long compother,long ipp, double x1,double x2,double x3,double x4);
  void print_othervalue_name(FILE *out,long compother);
  double pgws(double x1,double x2, double x3,double x4);
  double permeabilitavodnipary(double x1,double x2, double x3,double x4);
  void inverze_sorption_izoterm_data(double x1, double &fi, double &dfi);
  void sisotherm(int kod, double x1,double &fiw, double &dfdw);
  double linear_data(int kod, double x1,double x2, double x3,double x4);
  double kapa(int kod, double x1,double x2, double x3,double x4);
  double diffcoefiont(int kod, double x1,double x2, double x3,double x4);
  double mw, ma, gasr;
  void values_correction (vector & nv,long ipp);
  void Cf_check(double cf, double w, long ipp);
  void binding_izoterm_derivation(double x2, double & derbi);
  
  void kapa_values (int kod, long ipp,double x1, double xpv, double ineq1, double &kapa);
  void salt_diffusivity_values (int kod, long ipp, double x2, double xpv,double ineq1, double &diff);
  void binding_izoterm_derivation_values (int kod, long ipp, double x2, double xpv, double ineq1, double &cb, double &dcbdcf);
  void sorption_izoterms_values(int kod, long ipp, double x1,double xpv, double ineq1, double & fi, double & dfdw);
  
  
  void water_content_relhum (long nid,double *in,double *inp,double *ineq, double *out);
  double inverse_hystereze_sorption_izoterms (long ipp, double in, double inp, double ineq);
  void get_moisture (long nid,double in,double *inp, double &out);

  void initvalues (long ipp,long ido);

  void give_values (long ipp,double *av, double *pv, double *eq);
  void aux_values (long ipp,double *in,double *inp, double *ineq,double *out);
  void save_values (long ipp,double *out);
  void aux_values_elements (long ipp,double *in,double *inp, double *ineq,double *out);

  void der_value_hyst (int matchar,int kod, double pv, double & outvalue,double & outvalue2, long ipp);
  double derivation_saturation_water_vapour_pressure_temperature(double x1, double x2, double x3, double x4);
  double latent_heat_of_evaporation_of_water(double x1, double x2, double x3, double x4);
  void sorption_izotherm_derivation(double x1, double x2, double x3,double x4, double & derfi);
  //double get_moisture(double rh);
  void give_data_si_root_dfidw(double x1, double x2, double x3, double rh_hyg, double w_sat, double w_hyg, double shift_w, double & dfdw);
  double sortpion_isotherm_root_shifted(double x1, double w_hyg, double rh_hyg);
  double get_rel_hum(double w);
  void give_data_si_root_fi(double x1, double x2, double x3, double w_hyg, double w_sat, double rh_hyg, double shift_w,  double & relh);
  long cycle_detection (double *r,double *pr, double *ppr);
  double derivation_dy_dx (int matchar, double prom, int pomk1, int pomk2);
  
  void hystereze2 (int matchar, double x, double xpv, double ineq1,double & outvalue,double & outvalue2, long ipp, long timeH);
	double get_moisture2 ( double rh);
	double get_rel_hum2 (double w);
 private:
  int kd;
  double a1,a2,a3;
  int MatChar [20];		// popisuje pro jednotlivy material typ modelu a typ jednotlivych vlastnosti
  double MatConst [20];     // constanti hodnoty mat. vlastnosti
  double MatData [20][4][150]; // maximalni pocet 150 radku na jednu charakteristiku
  double MatFunce [20][5];   // charakteristika zadana nejakou funkci - 5 promennych
  void CorD(int cislochar, int &kvyhl,double in,int rhw, double x, double & y, double & z, double &z2);
  double Init[20];
  double k1[20],k2[20],k3[20],k4[20],k5[20],k6[20]; 
  
};

#endif

