#include "stochdrivert.h"
#include "matrix.h"
#include "vector.h"
#include "globalt.h"
#include "isotrmat.h"
#include "crsection1d.h"
#include "crsection2d.h"
#include "crsection3d.h"

stochdrivert::stochdrivert (void)
{
  nsmt=0;  nscs=0;
  nsampl=0;  nstochvar=0;  nprunknowns=0;
  mt=NULL;  idm=NULL;  atm=NULL;
  cst=NULL;  idcs=NULL;  atcs=NULL;
  
  nna=NULL;  pnv=NULL;
}

stochdrivert::~stochdrivert (void)
{
  delete [] mt;  delete [] idm;  delete [] atm;
  delete [] cst;  delete [] idcs;  delete [] atcs;
  delete [] nna;  delete [] pnv;
}

void stochdrivert::read (XFILE *in)
{
  long i;
  FILE *datin;
  
  getnextln(in->file);
  inputln (in->file,auxfile,1000);
  datin = fopen (auxfile,"r");
  
  inputln (in->file,auxfile,1000);
  
  xfscanf (in,"%ld",&nsmt);
  if (Mesprt==1)  fprintf (stdout,"\n number of stochastic materials  %ld",nsmt);
  if (nsmt>0){
    mt = new mattypet [nsmt];
    idm = new long [nsmt];
    atm = new atsel [nsmt];
    for (i=0;i<nsmt;i++){
      xfscanf (in,"%d %ld",(int*)mt+i,idm+i);
      idm[i]--;
      atm[i].read(in);
    }
  }
  
  xfscanf (in,"%ld",&nscs);
  if (Mesprt==1)  fprintf (stdout,"\n number of stochastic cross sections  %ld",nscs);
  if (nscs>0){
    cst = new crsectypet [nscs];
    idcs = new long [nscs];
    atcs = new atsel [nscs];
    for (i=0;i<nscs;i++){
      xfscanf (in,"%d %ld",(int*)cst+i,idcs+i);
      idcs[i]--;
      atcs[i].read(in);
    }
  }
  
  xfscanf (in,"%ld",&npnv);
  if (Mesprt==1)  fprintf (stdout,"\n number of printed nodal values  %ld",npnv);
  if (npnv>0){
    nna = new long [npnv];
    pnv = new atsel [npnv];
    for (i=0;i<npnv;i++){
      xfscanf (in,"%ld",nna+i);
      nna[i]--;
      pnv[i].read (in);
    }
  }
  
  
  readtable (datin);
  
  compute_nprunknowns ();
  
  fclose (datin);
}

void stochdrivert::compute_nprunknowns ()
{
  long i;

  nprunknowns=0;
  for (i=0;i<npnv;i++){
    nprunknowns+=pnv[i].num;
  }
  
  allocm (nsampl,nprunknowns,stochtabout);
}

/**
   function reads table of stochastic values
   number of rows = number of samples
   number of columns = number of stochastic variables in the problem
   
   13.3.2003
*/
void stochdrivert::readtable (FILE *in)
{
  fscanf (in,"%ld %ld",&nsampl,&nstochvar);
  allocm (nsampl,nstochvar,stochtabin);
  readm(in, stochtabin);
}

/**
   function reads table of stochastic values
   number of rows = number of samples
   number of columns = number of stochastic variables in the problem
   
   13.3.2003
*/
void stochdrivert::writetable ()
{
  FILE *out;
  out = fopen (auxfile,"w");

  fprintf (out,"%ld\n",nprunknowns);
  printm(stochtabout,out,10,20);
  
  fclose (out);
}

/**
   function changes stochastic values
   
   13.3.2003
*/
void stochdrivert::changevalues (long sampleid)
{
  long i,j,k,l,m;
  vector val;
  
  j=0;
  for (i=0;i<nsmt;i++){
    k=atm[i].num;  m=0;
    allocv (k,val);
    for (l=j;l<j+k;l++){
      val[m]=stochtabin[sampleid][l];
      m++;
    }
    j+=k;
    changematerials (i,val);
    destrv (val);
  }

  for (i=0;i<nscs;i++){
    k=atcs[i].num;  m=0;
    allocv (k,val);
    for (l=j;l<j+k;l++){
      val[m]=stochtabin[sampleid][l];
      m++;
    }
    j+=k;
    changecrsections (i,val);
    destrv (val);
  }

}

void stochdrivert::changematerials (long id,vector &val)
{
  switch (mt[id]){
  case isotransmat:{
    Tm->itrm[idm[id]].changeparam (atm[id],val);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown material type is required in function changematerials (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}

void stochdrivert::changecrsections (long id,vector &val)
{
  switch (cst[id]){
  case nocrosssectiont:{
    break;
  }
  case crsec1dt:{
    Tc->cs1d[idcs[id]].changeparam (atcs[id],val);
    break;
  }
  case crsec2dt:{
    Tc->cs2d[idcs[id]].changeparam (atcs[id],val);
    break;
  }
  case crsec3dt:{
    Tc->cs3d[idcs[id]].changeparam (atcs[id],val);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown material type is required in function changecrsections (file %s, line %d).\n",
	     __FILE__,__LINE__);
  }
  }
}

void stochdrivert::extractor (long sampleid)
{
  long i,j,k,ci,nid;
  
  ci=0;
  for (i=0;i<npnv;i++){
    nid=nna[i];
    for (j=0;j<pnv[i].num;j++){
      k=Tt->give_dof (nid,pnv[i].atrib[j])-1;
      if (k<0){
	fprintf (stderr,"\n\n wrong number of DOF in function extractor (file %s, line %d).\n",__FILE__,__LINE__);
      }
      stochtabout[sampleid][ci]=Lsrst->lhs[k];
      ci++;
    }
  }
}
