#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "trfelinit.h"
#include "globalt.h"
#include "globmatt.h"
#include "intpointst.h"
#include "gtopology.h"
#include "transprint.h"
#include "iotools.h"
#include "xfile.h"

/**
   trfel initialization
   function reads data from input file and initiates all
   necessary variables and arrays
   
   @param argv - optinal arguments
   @param stochd - pointer to %stochdrivert
   
*/
void trfel_init (int argc, const char *argv[], stochdrivert *stochd)
{
  long i,j;
  XFILE *in;

  fprintf (stderr,"\n\n ******************************************************\n");

  Tp = new probdesct;
  Gtt = new gtopology;
  Tt = new transtop;
  Tm = new transmat;
  Tc = new transcrsec;
  Tb = new transbclc;
  Outdt = new outdrivert;

  if (process_argst(argc, argv))
  {
    delete Tp;
    delete Gtt;
    delete Tt;
    delete Tm;
    delete Tc;
    delete Tb;
    delete Outdt;
    abort();
  }

  in = xfopen(argv[1],"r");
  if (in==NULL){
    fprintf (stderr,"\n Input file has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    abort ();
  }
  in->maxlnsize = 1000;
  in->warrning = 1;
  if ((Tp->kwdswt == nokwdt) || (Tp->kwdswt == kwd_outdt))
  in->kwdmode = ignore;
  else
  in->kwdmode = sequent_mode;
  //in->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  in->ignorecase = 1;


  Outt = fopen ("trfel.log","wt");  
  
  //  basic problem data
  Tp->read (in);
  
  if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    //  initial time setting
    Tp->time=Tp->timecont.starttime ();
  }
  
  if (Tp->kwdswt == kwd_fullt)
    in->kwdmode = sequent_mode;
  else
    in->kwdmode = ignore;
  
  //  input of node and element data
  Tt->read (in);
  
  // mesh control
  /*
  if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    if (Mesprt==1){
      merr=Tt->mesh_check();
      if (merr > 0){
	fprintf (stdout,"Mesh error = %ld\n\n\n",merr);
	exit(0);
      }
    }
  }
  */
  
  
  if (Tp->tprob == discont_nonstat_problem || Tp->tprob == discont_nonlin_nonstat_problem){
    Gtt->auto_subdom_detection (Outt);
    Tt->alloc_enodes ();
    Tt->alloc_edges ();
  }
  
  // *************************************
  //  generation of ordering of unknowns
  //  (generation of code numbers)
  //  number of degrees of freedom
  // *************************************
  Ndoft = Gtt->codenum_generation (Outt);
  if (Mesprt==1)
    fprintf (stdout,"\n number of degrees of freedom    %ld",Ndoft);
  
  if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    Gtt->read_gf (in);
  }

  //  input of material characteristics
  Tm->read (in);
  
  //int. points allocation and initiation
  Tm->intpointalloc ();
  Tm->intpointinit ();

  //  input of cross section characteristics
  Tc->read (in);
  
  //  input of boundary conditions
  Tb->read (in);

  //  prescribed values on elements investigation
  //Tt->elemprescvalue();
  //  searching for elements with nodes where source is defined
  Tb->elemsource ();
  
  //  global vectors allocations
  Lsrst = new lhsrhst;
  Lsrst->alloc ();
  Lsrst->initcond (in);
  
  if (Tp->stochasticcalc){
    stochd = new stochdrivert;
    stochd->read (in);
  };
  
  if (Tp->tprob == discont_nonstat_problem || Tp->tprob == discont_nonlin_nonstat_problem){

    Tt->enodes_init ();
    Tt->edge_init ();

    //  definition of material types and id on nodes
    //Tt->node_materials ();
    
  }
  //Tt->node_materials ();
  //  toto je pokusne umisteni


  if ((Tp->kwdswt == nokwdt) || (Tp->kwdswt == kwd_pdt))
    in->kwdmode = ignore;
  else
    in->kwdmode = sequent_mode;

  // input of output description
  Outdt->read(in);

  if (Tp->kwdswt == kwd_fullt)
    in->kwdmode = sequent_mode;
  else
    in->kwdmode = ignore;

  //fclose (Outt);

  /*
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  Tp->filename_decompositiont (Outdt->outfn,path,name,suffix);
  sprintf(fname, "%s%s.log", path, name);
  Outt = fopen(fname,"wt");
  delete [] path; delete [] name; delete [] suffix;
  */
  
  //  nodes allocation
  Tt->alloc_nodes ();
  
  
  
  if (Tp->ssle->tlinsol == sfeti){
    Gtt->stop->compute_multiplicity (Outt);
    Gtt->stop->find_boundary_nodes (Outt);
    Gtt->stop->rewrite_ltg ();
    
    long **fetinodes;
    fetinodes = new long* [Tp->ssle->feti.ns];
    for (i=0;i<Tp->ssle->feti.ns;i++){
      fetinodes[i] = new long [Gtt->stop->nnsd[i]];
    }
    
    for (i=0;i<Tp->ssle->feti.ns;i++){
      for (j=0;j<Gtt->stop->nnsd[i];j++){
	if (Gtt->stop->ltg[i][j]>-1)
	  fetinodes[i][j]=Gtt->stop->ltg[i][j];
	else
	  fetinodes[i][j]=-1;
      }
    }
    
    seqselnodes *selnodfeti;
    
    selnodfeti = new seqselnodes (Tp->ssle->feti.ns,Gtt->stop->nnsd,fetinodes,Gtt->stop->md,Outt,1);
    
    selnodfeti->prepare_feti (Gtt,Outt);

    //Tt->jump_initiation (Gtt->stop->tnbn,selnodfeti);
    //Tt->node_materials ();

    Tp->ssle->feti.initiate (selnodfeti,Gtt,Outt);

    Tp->ssle->feti.assemble_subdom_unknowns (Gtt,Outt);
    
    fprintf (Outt,"\n\n\n tnsn je %ld",selnodfeti->tnsn);
    fprintf (Outt,"\n\n\n tnbn je %ld",Gtt->stop->tnbn);
    //selnodfeti->~seqselnodes ();

    /****

     KDE JE DEALOKACE selnodfeti a fetinodes

    *****/
  }


  
  //if (Tp->tprob == discont_nonstat_problem){
  //Tt->node_materials ();
  //}
  
  
  xfclose (in);
}

void print_helpt(FILE *out)
{
  fflush(stderr);
  fprintf(stderr, "\nInput file has not been specified\n");
  fprintf(stderr, " use : trfel infile [-kwd=i]\n");
  fprintf(stderr, " where :\n *   infile is input file name\n");
  fprintf(stderr, " *   -kwd={0,1,2,3,4} is optional switch which controls keyword usage in the input file\n");
  fprintf(stderr, "           0 means no keywords (default value)\n");
  fprintf(stderr, "           1 means keywords only in the problem description section\n");
  fprintf(stderr, "           2 means keywords only in the outdriver section\n");
  fprintf(stderr, "           3 means keywords in the problem description and outdriver sections\n");
  fprintf(stderr, "           4 means full usage of keywords\n\n");
}


long process_argst(int argc, const char *argv[])
{
  const char *pkwd;
  
  switch (argc)
  {
    case 0:
    case 1:
      fflush(stdout);
      fprintf(stderr, "\n\nError - too few arguments is used\n\n");
      print_helpt(stderr);
      return 1;
    case 2:
      Tp->kwdswt = nokwdt;
      break;
    case 3:
      // processing of optional keyword switch -kwd={0,1,2,3,4}
      pkwd = strstrcis(argv[2], "-kwd=");
      if (pkwd)
      {
        Tp->kwdswt = pkwd_swt(-1);
        sscanf(pkwd+5, "%d", (int *)&Tp->kwdswt);
        if ((Tp->kwdswt < 0) && (Tp->kwdswt > 4))
	{ 
          fflush(stdout);
          fprintf(stderr, "\n\nError - unknown -kwd value is required\n\n");
          print_helpt(stderr);
          return 1;
        }
      }
      else
      {
        fflush(stdout);
        fprintf(stderr, "\n\nError - unknown switch is used\n\n");
        print_helpt(stderr);
        return 1;
      }
      break;
    default:
      fflush(stdout);
      fprintf(stderr, "\n\nError - too many arguments is used\n\n");
      print_helpt(stderr);
      return 1;
  }
  return 0;
}

